# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is ChangeNotification version 1.0.
#
# The Initial Owner of the Original Code is European Environment
# Agency (EEA).  Portions created by CMG and Finsiel Romania are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributor(s):
# Cornel Nitu, Finsiel Romania

#This product includes software developed by Jens Vagelpohl
#for use in the Z Object Publishing Environment
#(http://www.zope.org/).

from os.path import join
import string
import re

from OFS.ObjectManager import ObjectManager
from OFS.SimpleItem import SimpleItem
from Globals import DTMLFile, MessageDialog
from AccessControl import getSecurityManager, Permissions, ClassSecurityInfo
from AccessControl.User import nobody
from AccessControl.DTML import RestrictedDTML
from DocumentTemplate.DT_Util import InstanceDict,TemplateDict,Eval
from DateTime import DateTime
import Globals

import Products
from Sentinel import SentinelID
from FilterManager import FilterManager
from Utils import *

try:
        from Products.Xron import *
        XRONinstalled=1
except:
        XRONinstalled=0

import Subscriber

#constants
_NOTIFICATION_IMMEDIATE = 0
_NOTIFICATION_DAILY = 1
_NOTIFICATION_WEEKLY = 2

manage_addNotificationsFolderForm = DTMLFile('dtml/NotificationsFolder_addForm', globals())
def manage_addNotificationsFolder(self, id, title='', news_metatypes=[], upload_metatypes=[], folder_metatypes=[], site_title='', subject_line='Notifications', news_line='Newsletter', from_address='Postmaster', timeout=2, method_use='', __work_mode=1, modifications='', REQUEST=None):
        """Add a new NotificationsFolder object"""
        
        #data conversions
        news_metatypes = ConvertLinesToList(news_metatypes)
        upload_metatypes = ConvertLinesToList(upload_metatypes)
        folder_metatypes = ConvertLinesToList(folder_metatypes)

        notification_immediate = 0
        if REQUEST.has_key('notification_immediate'):
                notification_immediate = 1
        notification_daily = 0
        if REQUEST.has_key('notification_daily'):
                notification_daily = 1
        notification_weekly = 0
        if REQUEST.has_key('notification_weekly'):
                notification_weekly = 1

        ob = NotificationsFolder(id, title, news_metatypes,
                upload_metatypes, folder_metatypes , site_title,
                subject_line, news_line, from_address, timeout,
                method_use, __work_mode, modifications,
                notification_immediate, notification_daily,
                notification_weekly,REQUEST)
        ob.id = id
        self._setObject(id, ob)

        # set proxy_roles for Xron methods
        notif_ob = getattr(self, id)
        getattr(notif_ob,'daily_notifications').manage_proxy(['Manager'])
        getattr(notif_ob,'weekly_notifications').manage_proxy(['Manager'])

        mailheader = open(join(Globals.package_home(globals()),
            'dtml','MailHeader.dtml'))

        content = mailheader.read()
        mailheader.close()
        ob.manage_addDTMLMethod('headermail', title='Header for mail', file=content)

        mailfooter = open(join(Globals.package_home(globals()),
            'dtml','MailFooter.dtml'))
        content = mailfooter.read()
        mailfooter.close()
        ob.manage_addDTMLMethod('footermail', title='Footer for mail', file=content)

        mailcss = open(join(Globals.package_home(globals()),
            'dtml','MailCss.dtml'))
        content = mailcss.read()
        mailcss.close()
        ob.manage_addDTMLMethod('cssmail', title='Css for mail', file=content)

        mailconfirmation = open(join(Globals.package_home(globals()),
            'dtml','MailConfirmation.dtml'))
        content = mailconfirmation.read()
        mailconfirmation.close()
        ob.manage_addDTMLMethod('confirmationmail',
                title='Confirmation mail', file=content)

        css_forms = open(join(Globals.package_home(globals()),
            'dtml','CssForms.dtml'))
        content = css_forms.read()
        css_forms.close()
        ob.manage_addDTMLMethod('css_forms', title='css forms', file=content)

        #trigger object in Sentinel
        oSentinel = self.unrestrictedTraverse(SentinelID)
        oSentinel.addToList(computeRelativePath(self.absolute_url(1), id))

        if REQUEST is not None:
                return self.manage_main(self, REQUEST, update_menu=1)


class NotificationsFolder(ObjectManager,SimpleItem,FilterManager):
    """A folder-like product that can contain Subscribers"""

    meta_type="Notifications folder"
    product_name="ChangeNotifications"

    security=ClassSecurityInfo()

    security.setPermissionDefault('Add Subscriber for notifications','Anonymous')
    security.setPermissionDefault('Use mailhost services','Anonymous')

    security.declareProtected('View management screens', 'Sitemap')
    Sitemap = Globals.DTMLFile("dtml/Sitemap", globals())
    
    security.declarePublic('registration')
    registration = Globals.DTMLFile("dtml/RegistrationForm", globals())
    
    security.declarePrivate('listChoices')
    listChoices = Globals.DTMLFile("dtml/ListChoices", globals())
    
    security.declarePublic('index_html')
    index_html = Globals.DTMLFile("dtml/login", globals())
    
    security.declarePublic('message_form')
    message_form = Globals.DTMLFile("dtml/Message", globals())
    
    security.declareProtected('View management screens', 'emailSubscribers')
    emailSubscribers = Globals.DTMLFile("dtml/sendEmail", globals())
    
    security.declareProtected('View management screens', 'statistics')
    statistics = Globals.DTMLFile("dtml/NotificationsFolderIndex", globals())

    manage_filters_html = DTMLFile('dtml/NotificationsFolder_manage_filters', globals())
    
    security.declareProtected('View management screens', 'NotificationsFolder_properties')
    NotificationsFolder_properties = Globals.DTMLFile("dtml/NotificationsFolder_editForm", globals())

    #subscribers forms and constructors
    manage_addSubscriberForm=Subscriber.manage_addSubscriberForm
    manage_addSubscriber=Subscriber.manage_addSubscriber
    registerSubscriber = Subscriber.registerSubscriber

    manage_options = (ObjectManager.manage_options[0],) + (
                    ({'label' : 'Statistics','action' : 'statistics'},
                    {'label' : 'Preferences','action' : 'NotificationsFolder_properties'},
                    {'label' : 'Filters','action' : 'manage_filters_html'},
                    {'label' : 'View','action' : 'index_html'},
                    {'label' : 'Sitemap','action' : 'Sitemap'},
                    {'label' : 'Email subscribers','action' : 'emailSubscribers'},
                    {'label' : 'Undo','action' : 'manage_undoForm'},) + SimpleItem.manage_options)

    def all_meta_types(self):
            """ What can you put inside me? """
            f = lambda x: x['name'] in ('DTML Method', 'Page Template', 'Subscriber', 'XronDTMLMethods')
            return filter(f, Products.meta_types)

    def __init__(self,id, title, news_metatypes, upload_metatypes, folder_metatypes, site_title, subject_line, news_line, from_address, timeout, method_use, __work_mode, modifications, notification_immediate, notification_daily, notification_weekly, REQUEST=None):
            """Initialize a new instance of NotificationsFolder"""
            self.id = id
            self.title = title
            self.news_metatypes = news_metatypes
            self.upload_metatypes = upload_metatypes
            self.folder_metatypes = folder_metatypes
            self.site_title = site_title
            self.subject_line = subject_line
            self.news_line = news_line
            self.from_address = from_address
            self.timeout = int(timeout)
            self.method_use = method_use
            self.__work_mode = int(__work_mode)
            self.modifications = modifications
            self.notification_immediate = notification_immediate
            self.notification_daily = notification_daily
            self.notification_weekly = notification_weekly
            self.folders = []
            self.__links = {}
            self.__recent_uploads = []
            self.__recent_news = []
            FilterManager.__dict__['__init__'](self)
            
            if self.method_use == 'XronDTMLMethod':
                    XronDTMLMethod.manage_addXronDTMLMethod(self,'daily_notifications','Daily notifications',
                            file='Content-Type: text/plain\n\n<dtml-call dailyTrigger>',
                            executeAt= DateTime() + 1.0,
                            periodDays = 1.0)
                    
                    XronDTMLMethod.manage_addXronDTMLMethod(self,'weekly_notifications','Weekly notifications',
                            file='Content-Type: text/plain\n\n<dtml-call weeklyTrigger>',
                            executeAt= DateTime() + 7.0,
                            periodDays = 7.0)
            else:
                    self.manage_addDTMLMethod('daily_notifications', title='Daily notifications', file='<dtml-call dailyTrigger>')
                    self.manage_addDTMLMethod('weekly_notifications', title='Weekly notifications', file='<dtml-call weeklyTrigger>')
                    getattr(self,'daily_notifications').manage_permission('View',('Manager',),acquire=0)
                    getattr(self,'weekly_notifications').manage_permission('View',('Manager',),acquire=0)

    def __setstate__(self, state):
        """update"""
        NotificationsFolder.inheritedAttribute('__setstate__')(self, state)
        if not hasattr(self, '__filters'):
            self.__filters = {}

    security.declareProtected('View management screens','email_subscribers')
    def email_subscribers(self, REQUEST):
            """ Email all subscribers"""
            if REQUEST:
                    subs_list = self.getAllSubscribers()
                    if len(subs_list) == 0:
                            return MessageDialog(title = 'Send email',
                                    message = 'No subscribers found!',
                                    action = 'manage_main?update_menu=1',)
                    for subscriber in subs_list:
                            if subscriber.approved:
                                    eTo=subscriber.email
                                    eFrom=self.from_address
                                    eSubject=REQUEST['subject_line']
                                    eContent=REQUEST['content']
                                    subscriber.SendEmail(eContent, eTo, eFrom, eSubject)

                    return MessageDialog(title = 'Send email',
                            message = 'The email was send successfully!',
                            action = 'manage_main?update_menu=1',)


    security.declarePrivate('isobjSubscribable')
    def isobjSubscribable(self,obj_url):
            """ check if the object is in the notification's folders list"""
            if self.filterLinkForFolder(obj_url):
                    return 1
            return 0

    security.declarePublic('dailyTrigger')
    def dailyTrigger(self):
            """Daily trigger used to send notifications"""
            results=[]
            news_letter = []
            objects = []
            ROOT=self.unrestrictedTraverse(['',])
            if self.modifications == 'on':
                if len(self.upload_metatypes) != 0: 
                    objects=ROOT.PrincipiaFind(ROOT,obj_mtime=DateTime()-1,obj_metatypes=self.upload_metatypes,search_sub=1)
                    objects.extend(self.getRecentNews())
                else:
                    objects = self.getRecentNews()
            else:
                objects=self.getRecentUploads()
                objects.extend(self.getRecentNews())
            for object in objects:
                    if (string.find(object[0], 'Control_Panel') == -1):
                            if (object[1].meta_type in self.upload_metatypes):
                                    results.append(object[0])
                            if (object[1].meta_type in self.news_metatypes):
                                    news_letter.append(object[0])
            results=listIntersection(results, self.getLinks().get(ConvertDateTimeToString(DateTime()), []))
            self.SendNotifications(results, news_letter, _NOTIFICATION_DAILY)
            self.initRecentUploads()
            self.initRecentNews()
    
    security.declarePublic('weeklyTrigger')
    def weeklyTrigger(self):
            """Weekly trigger used to send notifications"""
            results = []
            for day_list in self.getLinks().values():
                    if len(day_list) > 0:
                            results.extend(day_list)
            self.SendNotifications(results, [], _NOTIFICATION_WEEKLY)
            self.initLinks()
                            
    security.declarePublic('getWorkMode')
    def getWorkMode(self):
            """Get for __work_mode"""
            return self.__work_mode

    security.declarePrivate('getLinks')
    def getLinks(self):
            """Get for __links"""
            return self.__links

    security.declarePrivate('getRecentUploads')
    def getRecentUploads(self):
        """Get for __recent_uploads"""
        results = []
        for object in self.__recent_uploads:
            try:
                self.restrictedTraverse(object[0])
                results.append(object)
            except:
                pass
        return results

    security.declarePrivate('initRecentUploads')
    def initRecentUploads(self):
        """ Init __recent_uploads"""
        self.__recent_uploads = []
        self._p_changed = 1

    security.declarePrivate('getRecentNews')
    def getRecentNews(self):
        """Get for __recent_news"""
        results = []
        for object in self.__recent_news:
            try:
                self.unrestrictedTraverse(object[0])
                results.append(object)
            except:
                pass
        return results

    security.declarePrivate('initRecentNews')
    def initRecentNews(self):
        """ Init __recent_news"""
        self.__recent_news = []
        self._p_changed = 1

    security.declarePrivate('initLinks')
    def initLinks(self):
            """Init __links"""
            self.__links = {}
            self._p_changed = 1

    security.declarePrivate('isNewsMetaType')
    def isNewsMetaType(self, metatype):
            """Test that specified metaype is in news_metatypes"""
            return (metatype in self.news_metatypes)
    
    security.declarePrivate('isUploadMetaType')        
    def isUploadMetaType(self, metatype):
        """Test that specified metaype is in upload_metatypes list"""
        return (metatype in self.upload_metatypes)

    security.declarePublic('addLink')
    def addLink(self, date, link, object):
            """Add a new link """
            #if the metatype of the added object is in upload_metatypes list then notify, else ignore it
            if self.isUploadMetaType(object.meta_type):
                            str_date = ConvertDateTimeToString(date)
                            self.__recent_uploads.append((link, object))
                            dict_links = self.getLinks()
                            if dict_links.has_key(str_date):
                                    dict_links[str_date].append(link)
                            else:
                                    dict_links[str_date] = []
                                    dict_links[str_date].append(link)
                            self._p_changed = 1

                            #send notification
                            self.SendNotifications([link], [], _NOTIFICATION_IMMEDIATE)
            elif self.isNewsMetaType(object.meta_type):
                self.__recent_news.append((link, object))
                self._p_changed = 1
            else:
                    #ignore this object
                    pass

    security.declarePrivate('getAllSubscribers')
    def getAllSubscribers(self):
            """Return a list with all Subscriber objects"""
            return self.objectValues('Subscriber')

    security.declareProtected('View management screens','managePreferences')
    def managePreferences(self, title='', news_metatypes=[], upload_metatypes=[], folder_metatypes=[], site_title='', subject_line='Notifications', news_line='Newsletter',from_address='Postmaster', timeout=2, modifications='', REQUEST=None):
            """Manage Notifications folder preferences"""
            news_metatypes = ConvertLinesToList(news_metatypes)
            upload_metatypes = ConvertLinesToList(upload_metatypes)
            folder_metatypes = ConvertLinesToList(folder_metatypes)
            self.title = title
            self.news_metatypes = news_metatypes
            self.upload_metatypes = upload_metatypes
            self.updateSiteMap(self.folder_metatypes, folder_metatypes)
            self.folder_metatypes = folder_metatypes
            self.site_title = site_title
            self.subject_line = subject_line
            self.news_line = news_line
            self.from_address = from_address
            self.timeout = int(timeout)
            self.modifications = modifications
            notification_immediate = 0
            if REQUEST.has_key('notification_immediate'):
                    self.notification_immediate = 1
            else:
                    self.notification_immediate = 0
            notification_daily = 0
            if REQUEST.has_key('notification_daily'):
                    self.notification_daily = 1
            else:
                    self.notification_daily = 0
            notification_weekly = 0
            if REQUEST.has_key('notification_weekly'):
                    self.notification_weekly = 1
            else:
                    self.notification_weekly = 0
            self._p_changed = 1
            REQUEST.RESPONSE.redirect('NotificationsFolder_properties')

    def manageFilters(self, REQUEST=None):
        """Manage Filter objects"""
        if addObjectAction(REQUEST):
            #add a new Filter object
            id = REQUEST.get('id', '')
            expr_filter = REQUEST.get('expr_filter', '')
            self.createFilter(id, expr_filter)
        elif updateObjectAction(REQUEST):
            #update a Filter object
            id = REQUEST.get('id', '')
            expr_filter = REQUEST.get('expr_filter', '')
            self.modifyFilter(id, expr_filter)
        elif deleteObjectAction(REQUEST):
            #delete Filter objects
            oIds = REQUEST.get('ids', [])
            self.deleteFilter(ConvertToList(oIds))
        REQUEST.RESPONSE.redirect('manage_filters_html')

    security.declarePrivate('testSpecificProperty')
    def testSpecificProperty(self, obj, obj_expr):
            """Test some propeties choosed by managers"""
            if obj_expr is not None:
                # Setup expr machinations
                print "intru"
                md=td()
                obj_expr=(Eval(obj_expr.expr_filter), md, md._push, md._pop)
                if expr_match(obj, obj_expr):
                    return 1
                else:
                    return 0
            else:
                return 1

    def manage_beforeDelete(self, item, container):
            """Update Sentinel list."""
            oSentinel = self.unrestrictedTraverse(SentinelID)
            oSentinel.delFromList(self.absolute_url(1))
            return

    security.declarePrivate('updateSiteMap')
    def updateSiteMap(self, old_folder_metatypes, new_folder_metatypes):
            """If the meta types list for objects in Sitemap is modified then update 'folders' and 'choices'"""
            del_folder_metatypes = listDifference(old_folder_metatypes, new_folder_metatypes)
            self.updateFoldersList(del_folder_metatypes)
            self.updateSubscribersChoices()

    security.declarePrivate('updateFoldersList')
    def updateFoldersList(self, folder_metatypes):
            """Delete all links from 'folders' with the meta type in 'folder_metatypes' list"""
            new_links = []
            for link in self.folders:
                    obj = self.unrestrictedTraverse(link)
                    if (obj.meta_type not in folder_metatypes):
                            new_links.append(link)
            self.folders = new_links
            self._p_changed = 1

    security.declareProtected('View management screens','updateSubscribersChoices')
    def updateSubscribersChoices(self):
            """Update Subscribers 'choices' list"""
            for subscriber_obj in self.getAllSubscribers():
                    del_choices = listDifference(subscriber_obj.getSubscriberChoicesList(), self.folders)
                    for item in del_choices:
                            subscriber_obj.delSubscriberChoice(item)
            self.REQUEST.RESPONSE.redirect(self.REQUEST.HTTP_REFERER)

    security.declareProtected('View management screens','folderAdd')
    def folderAdd(self):
            """Add a relative URL to the 'folders' list"""
            self.folders.append(self.REQUEST.PARENTS[1].absolute_url(1))
            self._p_changed = 1
            if self.REQUEST.form.has_key('back'):
                    self.REQUEST.RESPONSE.redirect(self.REQUEST.form['back'])
            else:
                    self.REQUEST.RESPONSE.redirect(self.REQUEST.HTTP_REFERER)
                    
    security.declareProtected('View management screens','folderRemove')
    def folderRemove(self):
            """Remove a relative URL from the 'folders' list"""
            self.folders.remove(self.REQUEST.PARENTS[1].absolute_url(1))
            self._p_changed = 1
            if self.REQUEST.form.has_key('back'):
                    self.REQUEST.RESPONSE.redirect(self.REQUEST.form['back'])
            else:
                    self.REQUEST.RESPONSE.redirect(self.REQUEST.HTTP_REFERER)

    security.declareProtected('View management screens','getChoices')
    def getChoices(self):
            """Returns a dictionary with 'absolute_url', 'name' (title_or_id), 'description' (if available)"""
            out=[]
            for item in self.folders:
                    crt={}
                    crt['absolute_url']=item
                    obj=self.unrestrictedTraverse(item)
                    crt['name']=obj.title_or_id()
                    if obj.hasProperty('description'):
                            crt['description']=obj.description
                    out.append(crt)
            return out

    security.declarePrivate('parseEmail')
    def parseEmail(self,email):
            """ parse an email"""
            transtab=string.maketrans('_','_')
            idx=string.translate(str(email),transtab,deletions='@$-_?&!:;.()<=>*#[]{}')
            return idx
    
    security.declarePublic('activateUser')
    def activateUser(self, REQUEST=None):
            """ activate an user"""
            transtab=string.maketrans('_','_')
            if REQUEST.has_key('user') and REQUEST.has_key('key'):
                foundUser = 0
                idx=string.translate(str(REQUEST['user']),transtab,deletions='@$-_?&!:;.()<=>*#[]{}')
                for user in self.objectValues('Subscriber'):
                        if (user.id == idx) and (user.get_confirmation_key() == str(REQUEST['key'])):
                            foundUser = 1
                            if REQUEST.has_key('active') and REQUEST['active'] == '0':
                                    user.setUserAccount(approved=0)
                            else:
                                    user.setUserAccount(approved=1)
                if foundUser == 0:
                    return self.REQUEST.RESPONSE.redirect('message_form?error=0')
    
    security.declarePrivate('ifSubscriber')
    def ifSubscriber(self,userid):
            """ check if subscriber exists"""
            for user in self.objectValues('Subscriber'):
                    if userid == user.id:
                            return user
            return None

    security.declarePublic('loginUser')
    def loginUser(self,REQUEST=None):
            """ login an user """
            if REQUEST is not None:
                    username = REQUEST.get('__name__', None)
                    password = REQUEST.get('__password__', None)
                    if username and password:
                            subscriber = self.ifSubscriber(self.parseEmail(username))
                            if subscriber is not None:
                                    if password == subscriber.getSubscriberPassword():
                                            REQUEST.SESSION.set('userkey',username)
                                            redirect_string = self.parseEmail(username)+'/index_html'
                                            if REQUEST.QUERY_STRING:
                                                redirect_string = redirect_string + '?' + REQUEST.QUERY_STRING
                                            REQUEST.RESPONSE.redirect(redirect_string)
                                    else:
                                            REQUEST.RESPONSE.redirect('index_html?error=1')
                            else:
                                    REQUEST.RESPONSE.redirect('index_html?error=1')

    security.declarePublic('logoutUser')
    def logoutUser(self,REQUEST=None):
            """ logout an user """
            if REQUEST is not None:
                    if REQUEST.SESSION.has_key('userkey'):
                            REQUEST.SESSION.delete('userkey')
                    REQUEST.RESPONSE.redirect('index_html')
                            
    security.declarePublic('mailPassword')
    def mailPassword(self, email,REQUEST=None, RESPONSE=None):
            """ Mail the password to subscriber"""
            transtab=string.maketrans('_','_')
            userid=string.translate(email,transtab,'?@-&!:;.()<=>*#[]{}')
            if self.ifSubscriber(userid) is not None:
                    eTo=email
                    eFrom=self.from_address
                    eSubject=self.subject_line + '--mailling list reminder'
                    eContent="YOUR PASSWORD IS:  " + self[userid].getSubscriberPassword()
                    self[userid].SendEmail(eContent, eTo, eFrom, eSubject)
                    if REQUEST is not None:
                            self.REQUEST.RESPONSE.redirect('message_form?forgotpasswd=1')
            else:
                    if REQUEST is not None:
                            self.REQUEST.RESPONSE.redirect('message_form?error=1')

    #################
    #   FILTERS     #
    #################
    
    security.declarePrivate('filterLinksForFolder')
    def filterLinksForFolder(self, links_list=[]):
            """Filter a list of relative links against the 'folders' list of a NotificationsFolder"""
            return filter(self.filterLinkForFolder, links_list)

    security.declarePrivate('filterLinkForFolder')
    def filterLinkForFolder(self, link):
            """Test if we have a valid link"""
            res = 0
            for local_folder in self.folders:
                    if ((local_folder == link) or (re.search('\\b' + local_folder + '/\\b', link))):
                            res = 1
                            break
            return res

    security.declarePrivate('handleSubscriber')
    def handleSubscriber(self, subscriber):
            """Test if specified subscriber is approved.
                            The method will return :
                                    -1 : subscriber deleted
                                     0 : subscriber ignored
                                     1 : subscriber ok"""
            res = -1
            if (subscriber.approved):
                    #this subscriber is ok
                    res = 1
            else:
                    if (subscriber.bobobase_modification_time().lessThanEqualTo(DateTime()-self.timeout)):
                            #timeout is over - delete subscriber
                            self.manage_delObjects(subscriber.id)
                    else:
                            #timeout is still ticking .. - ignore this subscriber for the moment
                            res = 0
            return res

    security.declarePrivate('getObjectTitle')
    def getObjectTitle(self, obj_relative_link):
            """Returns object's title"""
            try:
                    return self.unrestrictedTraverse(obj_relative_link, None).title_or_id()
            except:
                    return ''
                    
    security.declarePrivate('CreateLinksContent')
    def CreateLinksContent(self, html_content, links_list):
            """Build email content using the links list"""
            import types
            new_line = '\n'
            if html_content:
                    new_line = '<br>'
            content = new_line
            for link in links_list:
                    full_link = self.REQUEST.get('SERVER_URL', '') + '/' + str(link)
                    title_link = self.getObjectTitle(str(link))
                    if html_content:
                            content = content + new_line + '<a href="' + full_link + '">' + title_link + '</a>'
                    else:
                            content = content + new_line + title_link + ' - ' + full_link
            content = content + new_line
            return content
            
    def isNotificationImmediate(self, notification_type):
            """test for an immediate notification"""
            return (notification_type == _NOTIFICATION_IMMEDIATE)

    def isNotificationDaily(self, notification_type):
            """test for a daily notification"""
            return (notification_type == _NOTIFICATION_DAILY)

    def isNotificationWeekly(self, notification_type):
            """test for a weekly notification"""
            return (notification_type == _NOTIFICATION_WEEKLY)

    security.declarePrivate('SendNotifications')
    def SendNotifications(self, links_list,news_list, notification_type):
            """Send notifications to subscribers"""
            import types
            user = nobody
            folder_filtered_links_list = self.filterLinksForFolder(links_list)

            #step 2: loop through subscribers list and send emails
            for subscriber in self.getAllSubscribers():
                    #step 2.1: first get subscriber status
                    if self.handleSubscriber(subscriber) == 1:
                            #subscriber is ok !
                            #get Zope user
                            filtred_news_list = []
                            for news in news_list:
                                    try:
                                        news_obj = self.unrestrictedTraverse(news)
                                        if user.has_permission('View', news_obj):
                                                if self.testSpecificProperty(news_obj,self.getFilter(news_obj.meta_type)):
                                                        filtred_news_list.append(news)
                                    except:
                                        pass 
                            #send newsletter
                            if (subscriber.receive_news and len(filtred_news_list)>0):

                                    #step 2.1.0.1: create content
                                    subscriber_email_content = self.headermail(None, self.REQUEST,htmlcontent=subscriber.htmlcontent)
                                    subscriber_email_content = subscriber_email_content + self.CreateLinksContent(subscriber.htmlcontent, filtred_news_list)
                                    subscriber_email_content = subscriber_email_content + self.footermail(None, self.REQUEST,htmlcontent=subscriber.htmlcontent,email=subscriber.email,obj_url=subscriber.absolute_url(1),key=subscriber.get_confirmation_key(),parent=subscriber.aq_parent.id)
                                    #step 2.1.0.2: send email
                                    subscriber.SendEmail(subscriber_email_content, subscriber.email, self.from_address, self.news_line)
                            
                            #step 2.1.1: first filter link list for this subscriber
                            subscriber_filtered_links_list = subscriber.filterLinksForSubscriber(folder_filtered_links_list)

                            #step 2.1.2: filter links list by notification_type
                            subscriber_filtered_links_list = subscriber.filterLinksForSubscriberNotificationType(subscriber_filtered_links_list, notification_type)

                            filtred_links_list = []
                            for subscriber_filtered_link in subscriber_filtered_links_list:
                                                try:
                                                    link_obj = self.unrestrictedTraverse(subscriber_filtered_link)
                                                    if user.has_permission('View', link_obj):
                                                            if self.testSpecificProperty(link_obj, self.getFilter(link_obj.meta_type)):
                                                                filtred_links_list.append(subscriber_filtered_link)
                                                except:
                                                    pass
                            subscriber_filtered_links_list = filtred_links_list
                            if len(subscriber_filtered_links_list) > 0:
                                    #the filtered links list is not empty - send email
                                    
                                    #step 2.1.2.1: create content
                                    subscriber_email_content = self.headermail(None, self.REQUEST, htmlcontent=subscriber.htmlcontent)
                                    subscriber_email_content = subscriber_email_content + self.CreateLinksContent(subscriber.htmlcontent, filtred_links_list)
                                    subscriber_email_content = subscriber_email_content + self.footermail(None, self.REQUEST,htmlcontent=subscriber.htmlcontent,email=subscriber.email,obj_url=subscriber.absolute_url(1),key=subscriber.get_confirmation_key(),parent=subscriber.aq_parent.id)

                                    #step 2.1.2.2: send email
                                    subscriber.SendEmail(subscriber_email_content, subscriber.email, self.from_address, self.subject_line)
    

    #################
    #       TOOLS   # 
    #################
    def getSubscribersDetails(self):
        """ Statistics for the subscribed users """
        active_users = {}
        inactive_users = {}
        newsletter_users = 0
        for obj in self.objectValues('Subscriber'):
            if obj.approved:
                active_users[obj.id] = [obj.email, obj.firstname + ' ' + obj.lastname]
                if obj.receive_news:
                    newsletter_users = newsletter_users + 1
            else:
                inactive_users[obj.id] = [obj.email, obj.firstname + ' ' + obj.lastname]
        return active_users,inactive_users,newsletter_users

    ConvertListToLines = ConvertListToLines
    HTMLEncode = HTMLEncode
    URLEncode = URLEncode

# Initialize the class in order the security assertions be taken into account
Globals.InitializeClass(NotificationsFolder)

class td(RestrictedDTML, TemplateDict):
    pass

def expr_match(ob, ed, c=InstanceDict, r=0):
    e, md, push, pop=ed
    push(c(ob, md))
    try: r=e.eval(md)
    finally:
        pop()
        return r
