# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is ChangeNotification version 1.0.
#
# The Initial Owner of the Original Code is European Environment
# Agency (EEA).  Portions created by CMG and Finsiel Romania are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributor(s):
# Cornel Nitu, Finsiel Romania

#This product includes software developed by Jens Vagelpohl
#for use in the Z Object Publishing Environment
#(http://www.zope.org/).


import string
import re
from os.path import join

from OFS.SimpleItem import SimpleItem
from Globals import DTMLFile, InitializeClass
from AccessControl import ClassSecurityInfo
import Globals

from Utils import *
from MaildropHost import MaildropHost


manage_addSubscriberForm=DTMLFile('dtml/Subscriber_addForm', globals())
def manage_addSubscriber(self, id='', email='', firstname='', lastname='', htmlcontent=0, receive_news=0, REQUEST=None, RESPONSE=None):
    """Add a new Subscriber object"""

    if not email:
        return getattr(self, 'manage_addSubscriberForm')(setFormError(REQUEST, 'email', 'Please enter your email address!'))
    if email and not isEmailValid(email):
        return getattr(self, 'manage_addSubscriberForm')(setFormError(REQUEST, 'email', 'Your email address is invalid!'))

    transtab=string.maketrans('_','_')
    id=string.translate(email,transtab,'?@-&!:;.()<=>*#[]{}')
    key=generatePassword(20)
    user_pass=generatePassword(5)
    ob=Subscriber(id, email, firstname, lastname, htmlcontent, receive_news, key, user_pass, REQUEST)
    try:
        self._setObject(id, ob)
    except:
        return getattr(self,REQUEST['form_name'])(setFormError(REQUEST, 'user_exists', 'There already is a subscriber registered with this email address!'))
    obj_url=self.absolute_url(1)
    eContent=self.confirmationmail(None,self.REQUEST,email=email,password=user_pass,obj_url=obj_url,key=key,user=email,htmlcontent=ob.htmlcontent)
    eTo=email
    eFrom=self.from_address
    eSubject=self.subject_line
    ob.SendEmail(eContent, eTo, eFrom, eSubject)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)

def registerSubscriber(self, id='', email='', firstname='', lastname='', htmlcontent=0, receive_news=0, REQUEST=None, RESPONSE=None):
    """ Register a new Subscriber """
    if not email:
        return getattr(self, 'registration')(setFormError(REQUEST, 'email', 'Please enter your email address!'))
    if email and not isEmailValid(email):
        return getattr(self, 'registration')(setFormError(REQUEST, 'email', 'Your email address is invalid!'))

    transtab=string.maketrans('_','_')
    id=string.translate(email,transtab,'?@-&!:;.()<=>*#[]{}')
    key=generatePassword(20)
    user_pass=generatePassword(5)
    ob=Subscriber(id, email, firstname, lastname, htmlcontent, receive_news, key, user_pass, REQUEST)
    ob.id = id
    try:
        self._setObject(id, ob)
    except:
        return getattr(self,'registration')(setFormError(REQUEST, 'user_exists', 'There already is a subscriber registered with this email address!'))

    obj_url=self.absolute_url(1)
    eContent=self.confirmationmail(None,self.REQUEST,email=email,password=user_pass,obj_url=obj_url,key=key,user=email,htmlcontent=ob.htmlcontent)
    eTo=email
    eFrom=self.from_address
    eSubject="Subscription confirmation"
    ob.SendEmail(eContent, eTo, eFrom, eSubject)
    #handle preferences
    ob.updateSubscriberPreferences(self, REQUEST)
    if REQUEST is not None:
        REQUEST.RESPONSE.redirect('message_form?added=1')


class Subscriber(SimpleItem):
    """An object that contain relevant information about subscribers"""

    meta_type="Subscriber"
    product_name="ChangeNotifications"

    security=ClassSecurityInfo()
    security.declareProtected('View management screens', 'Subscriber_properties')
    Subscriber_properties = DTMLFile('dtml/Subscriber_editForm', globals())

    security.declarePublic('Subscriber_preferences')
    index_html = DTMLFile('dtml/Subscriber_preferences', globals())

    security.declarePublic('Subscriber_advanced_preferences')
    Subscriber_advanced_preferences = DTMLFile('dtml/Subscriber_advanced_preferences', globals())

    manage_options = (
            ({'label' : 'Properties', 'action' : 'Subscriber_properties'},)
            +
            SimpleItem.manage_options
    )



    def __init__(self, id, email, firstname, lastname, htmlcontent, receive_news, key, passwd, REQUEST=None):
        """Initialize a new instance of Subscriber"""

        self.id=id
        self.email=email
        self.firstname=firstname
        self.lastname=lastname
        self.approved=0
        self.htmlcontent=int(htmlcontent)
        if receive_news:
            self.receive_news=1
        else:
            self.receive_news=0
        self.__choices = {}
        self.__immediate_choices = []
        self.__daily_choices = []
        self.__weekly_choices = []
        self.__conf_key=key
        self.__pass=passwd

    def getSubscriberChoices(self):
        """Get __choices'"""
        return self.__choices

    def getSubscriberPassword(self):
        """Get __pass"""
        return self.__pass

    def get_confirmation_key(self):
        """ Get __conf_key"""
        return str(self.__conf_key)

    def getSubscriberChoicePreferences(self, value):
        """Returns the preferences for this choice"""
        return self.getSubscriberChoices()[value]

    def setSubscriberChoicePreferences(self, value, choices):
        """Returns the preferences for this choice"""
        self.getSubscriberChoices()[value] = choices

    def getSubscriberChoicesList(self):
        """Get a list with all choices"""
        return self.getSubscriberChoices().keys()

    def getSubscriberImmediateChoices(self):
        """Get '__immediate_choices'"""
        return self.__immediate_choices

    def getSubscriberDailyChoices(self):
        """Get '__daily_choices'"""
        return self.__daily_choices

    def getSubscriberWeeklyChoices(self):
        """Get '__weekly_choices'"""
        return self.__weekly_choices

    def isSubscriberChoice(self, value):
        """Test if a value is a choice"""
        return self.getSubscriberChoices().has_key(value)

    def getSpecificNotificationsList(self, notification_type):
        """Return a list"""
        if self.aq_parent.isNotificationImmediate(notification_type):
            return self.getSubscriberImmediateChoices()
        elif self.aq_parent.isNotificationDaily(notification_type):
            return self.getSubscriberDailyChoices()
        elif self.aq_parent.isNotificationWeekly(notification_type):
            return self.getSubscriberWeeklyChoices()
        else:
            return []

    def addChoice(self, REQUEST=None):
        """Add a choice"""
        if REQUEST:
            choice = REQUEST.get('obj_url', '')
            if choice != '':
                self.addSubscriberChoice(choice, self.aq_parent.notification_immediate, self.aq_parent.notification_daily, self.aq_parent.notification_weekly)

    def addSubscriberChoice(self, value, b_immediate, b_daily, b_weekly):
        """Add a new choice"""
        self.getSubscriberChoices()[value] = [int(b_immediate), int(b_daily), int(b_weekly)]
        if b_immediate:
            self.getSubscriberImmediateChoices().append(value)
        if b_daily:
            self.getSubscriberDailyChoices().append(value)
        if b_weekly:
            self.getSubscriberWeeklyChoices().append(value)
        self._p_changed = 1

    def updSubscriberChoice(self, value, b_immediate, b_daily, b_weekly):
        """Update a choice"""
        l_choice = self.getSubscriberChoicePreferences(value)
        if l_choice[0] != b_immediate:
            #update
            if l_choice[0] and not b_immediate:
                self.getSubscriberImmediateChoices().remove(value)
            if not l_choice[0] and b_immediate:
                self.getSubscriberImmediateChoices().append(value)
        if l_choice[1] != b_daily:
            #update
            if l_choice[1] and not b_daily:
                self.getSubscriberDailyChoices().remove(value)
            if not l_choice[1] and b_daily:
                self.getSubscriberDailyChoices().append(value)
        if l_choice[2] != b_weekly:
            #update
            if l_choice[2] and not b_weekly:
                self.getSubscriberWeeklyChoices().remove(value)
            if not l_choice[2] and b_weekly:
                self.getSubscriberWeeklyChoices().append(value)
        self.setSubscriberChoicePreferences(value, [b_immediate, b_daily, b_weekly])
        self._p_changed = 1

    def delSubscriberChoice(self, value):
        """Delete a choice"""
        l_choice = self.getSubscriberChoicePreferences(value)
        del(self.getSubscriberChoices()[value])
        if l_choice[0]:
            self.getSubscriberImmediateChoices().remove(value)
        if l_choice[1]:
            self.getSubscriberDailyChoices().remove(value)
        if l_choice[2]:
            self.getSubscriberWeeklyChoices().remove(value)
        self._p_changed = 1

    def manageSubscriberProperties(self, REQUEST=None):
        """Manage subscriber properties"""
        self.firstname = REQUEST.get('firstname', '')
        self.lastname = REQUEST.get('lastname', '')
        if REQUEST.has_key('approved'):
            self.approved = 1
        else:
            self.approved = 0
        if REQUEST.has_key('htmlcontent'):
            self.htmlcontent = 1
        else:
            self.htmlcontent = 0
        if REQUEST.has_key('receive_news'):
            self.receive_news = 1
        else:
            self.receive_news = 0
        self._p_changed = 1
        REQUEST.RESPONSE.redirect('Subscriber_properties')

    security.declarePrivate('setUserAccount')
    def setUserAccount(self,approved=0,RESPONSE=None):
        """ activate/deactivate an user """
        if approved==0:
            self.approved=0
            return self.REQUEST.RESPONSE.redirect('message_form?activate=0')
        else:
            self.approved=1
            return self.REQUEST.RESPONSE.redirect('message_form?activate=1')

    def manageSubscriberProfile(self, REQUEST=None):
        """Update subscriber profile (personal data & preferences)"""
        self.firstname = REQUEST.get('firstname', '')
        self.lastname = REQUEST.get('lastname', '')
        if REQUEST.has_key('htmlcontent'):
            self.htmlcontent = 1
        else:
            self.htmlcontent = 0
        if REQUEST.has_key('receive_news'):
            self.receive_news = 1
        else:
            self.receive_news = 0

        if not REQUEST['__password__'] and not REQUEST['confirmation']:
            pass
        else:
            if not REQUEST['__password__'] or not REQUEST['confirmation']:
                return getattr(self, 'index_html')(setFormError(REQUEST, 'not_pass', 'Password and confirmation must be specified'))

            if (REQUEST['__password__'] or REQUEST['confirmation']) and (REQUEST['__password__'] != REQUEST['confirmation']):
                return getattr(self, 'index_html')(setFormError(REQUEST, 'error_pass', 'Password and confirmation do not match'))

            if REQUEST['__password__'] == REQUEST['confirmation']:
                self.__pass = REQUEST['__password__']

        self.updateSubscriberPreferences(self.aq_parent, REQUEST)
        self._p_changed = 1
        REQUEST.RESPONSE.redirect('message_form?update=1')

    def updateSubscriberPreferences(self, parent_folder=None, REQUEST=None):
        """Update subscriber preferences"""
        if parent_folder is None:
            parent_folder = self.aq_parent

        b_notification_immediate = parent_folder.notification_immediate
        b_notification_daily = parent_folder.notification_daily
        b_notification_weekly = parent_folder.notification_weekly

        subscriber_choices = self.getSubscriberChoicesList()
        choices = ConvertToList(REQUEST.get('choice', []))

        add_choices = listDifference(choices, subscriber_choices)
        del_choices = listDifference(subscriber_choices, choices)

        #if notification folder is set on 'work mode ==2'
        if parent_folder.getWorkMode() ==2:
            for item in subscriber_choices:
                b_notification_immediate = int(REQUEST.get(str(item) + '_notification_immediate', 0))
                b_notification_daily = int(REQUEST.get(str(item) + '_notification_daily', 0))
                b_notification_weekly = int(REQUEST.get(str(item) + '_notification_weekly', 0))
                self.addSubscriberChoice(item, b_notification_immediate, b_notification_daily, b_notification_weekly)

        # step 1 : add choices
        for item in add_choices:
            if parent_folder.getWorkMode() == 2:
                b_notification_immediate = int(REQUEST.get(str(item) + '_notification_immediate', 0))
                b_notification_daily = int(REQUEST.get(str(item) + '_notification_daily', 0))
                b_notification_weekly = int(REQUEST.get(str(item) + '_notification_weekly', 0))
            self.addSubscriberChoice(item, b_notification_immediate, b_notification_daily, b_notification_weekly)
        # step 2 : del choices
        for item in del_choices:
            self.delSubscriberChoice(item)
        self._p_changed = 1

    def manageSubscriberAdvancedPreferences(self, REQUEST=None):
        """Update subscriber preferences"""
        b_notification_immediate = self.aq_parent.notification_immediate
        b_notification_daily = self.aq_parent.notification_daily
        b_notification_weekly = self.aq_parent.notification_weekly

        subscriber_choices = self.getSubscriberChoicesList()
        choices = ConvertToList(REQUEST.get('choice', []))
        add_choices = listDifference(choices, subscriber_choices)
        del_choices = listDifference(subscriber_choices, choices)
        upd_choices = listIntersection(choices, subscriber_choices)

        # step 1 : add choices
        for item in add_choices:
            b_item_notification_immediate = REQUEST.get(str(item) + '_notification_immediate', 0)
            b_item_notification_daily = REQUEST.get(str(item) + '_notification_daily', 0)
            b_item_notification_weekly = REQUEST.get(str(item) + '_notification_weekly', 0)
            self.addSubscriberChoice(item, b_item_notification_immediate, b_item_notification_daily, b_item_notification_weekly)

        # step 2 : del choices
        for item in del_choices:
            self.delSubscriberChoice(item)

        # ste p : update choices
        for item in upd_choices:
            b_item_notification_immediate = REQUEST.get(str(item) + '_notification_immediate', 0)
            b_item_notification_daily = REQUEST.get(str(item) + '_notification_daily', 0)
            b_item_notification_weekly = REQUEST.get(str(item) + '_notification_weekly', 0)
            self.updSubscriberChoice(item, b_item_notification_immediate, b_item_notification_daily, b_item_notification_weekly)

        REQUEST.RESPONSE.redirect('Subscriber_advanced_preferences')

    security.declarePrivate('SendEmail')
    def SendEmail(self, eContent, eTo, eFrom, eSubject):
        """Sends an email"""
        headers = {}
        headers['from'] = eFrom
        headers['to'] = eTo

        maildrop = MaildropHost()
        message = createmail(eFrom, eTo,eSubject,eContent,self.htmlcontent)
        maildrop._send(headers, message)
        return 1

    #---------Filters-----------------#     
    def filterLinksForSubscriber(self, links_list=[]):
        """Filter a list of relative links against the 'choices' list of a Subscriber"""
        return filter(self.filterLinkForSubscriber, links_list)

    def filterLinkForSubscriber(self, link):
        """Test if we have a valid link"""
        res = 0
        for local_folder in self.getSubscriberChoicesList():
            if ((local_folder == link) or (re.search('\\b' + local_folder + '/\\b', link))):
                res = 1
                break
        return res

    def filterLinksForSubscriberNotificationType(self, links_list=[], notification_type=-1):
        """Filter a list of relative links by notification type"""
        local_folders_list = self.getSpecificNotificationsList(notification_type)
        new_links_list = []
        for link in links_list:
            for local_folder in local_folders_list:
                if ((local_folder == link) or (re.search('\\b' + local_folder + '/\\b', link))):
                    new_links_list.append(link)
                    break
        return new_links_list

# Initialize the class in order the security assertions be taken into account
InitializeClass(Subscriber)