# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is ChangeNotification version 1.0.
#
# The Initial Owner of the Original Code is European Environment
# Agency (EEA).  Portions created by CMG and Finsiel Romania are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributor(s):
# Cornel Nitu, Finsiel Romania

#This product includes software developed by Jens Vagelpohl
#for use in the Z Object Publishing Environment
#(http://www.zope.org/).


import string
import re
import types

from Products.PythonScripts.standard import url_quote, html_quote


def setFormError(req, key, msg):
    req.set('FORM_ERROR', 1)
    req.set('FORM_ERROR_' + key, msg)	
    return req

def isEmailValid(email, bad_domains=''):
    if type(bad_domains) == type([]) and len(bad_domains) > 0:
        for dom in bad_domains:
            if re.compile('@'+dom).search(email) is not None:
                return 0

    if re.compile('\s').search(email) is not None:
        return 0

    if re.compile(r'^[_\-\.0-9a-z]+@([0-9a-z][_0-9a-z\.]+)\.([a-z]{2,4}$)', re.IGNORECASE).search(email) is None:
        return 0

    return 1

def generatePassword(lung):
    """ generate password """
    import random
    results = ""
    validchr = string.capitalize(string.letters) + string.digits
    length = len(validchr)
    for i in range(0,int(lung)):
        randnumber = int(length * random.random()) + 1
        results = results + validchr[randnumber-1]
    return results

def createmail(fromA, toA, subject, content, htmlcontent):
    """Create a mime-message
        'htmlcontent' may be 0/1
    """
    import MimeWriter
    import mimetools
    import cStringIO

    out = cStringIO.StringIO() # output buffer for our message 
    lin = cStringIO.StringIO(content)

    writer = MimeWriter.MimeWriter(out)
    writer.addheader("From", fromA)
    writer.addheader("To", toA)
    writer.addheader("Subject", subject)
    writer.addheader("MIME-Version", "1.0")

    if htmlcontent:
        type="html"
    else:
        type="plain"
        
    pout = writer.startbody("text/"+type, [("charset", 'iso-8859-1')])
    mimetools.encode(lin, pout, '8bit')
    lin.close()

    msg = out.getvalue()
    out.close()

    return msg

def computeRelativePath(url, id):
    """Build a relative path"""
    res = ''
    if url == '':
        res = str(id)
    else:
        res = str(url) + '/' + str(id)
    return res

def ConvertToList(something):
    """Convert to list"""
    ret = something
    if type(something) is type(''):
        ret = [something]
    return ret

def ConvertLinesToList(value):
    """Takes a value from a textarea control and returns a list of values"""
    if type(value) == type([]):
        return value
    if value == '':
        return []
    else:
        values = []
        for v in string.split(value , '\r\n'):
            if v != '':
                values.append(v)
    return values

def ConvertListToLines(self, values):
    """Takes a list of values and returns a value for a textarea control"""
    if len(values) == 0:
        return ''
    else:
        return string.join(values, '\r\n')

def listDifference(l1, l2):
    """return a list with elements from l1 that are not in l2"""
    return [e1 for e1 in l1 if e1 not in l2]

def listIntersection(l1, l2):
    """return the intersection of two lists"""
    return [e1 for e1 in l1 if e1 in l2]

def HTMLEncode(self, str):
    """Encode a string using html_quote"""
    return html_quote(str)

def URLEncode(self, str):
    """Encode a string using url_encode"""
    return url_quote(str)

def ConvertDateTimeToString(date):
    """Convert a DateTime object to a string 'mm/dd/yyyy'"""
    return (str(date.month()) + '/' + str(date.day()) + '/' + str(date.year()))


def addObjectAction(REQUEST=None):
    """Check if adding an object"""
    res = 0
    if REQUEST:
        res = REQUEST.has_key('add')
    return res

def updateObjectAction(REQUEST=None):
    """Check if updating an object"""
    res = 0
    if REQUEST:
        res = REQUEST.has_key('update')
    return res

def deleteObjectAction(REQUEST=None):
    """Check if deleting an object"""
    res = 0
    if REQUEST:
        res = REQUEST.has_key('delete')
    return res