# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is ChangeNotification version 1.0.
#
# The Initial Owner of the Original Code is European Environment
# Agency (EEA).  Portions created by CMG and Finsiel Romania are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributor(s):
# Cornel Nitu, Finsiel Romania

#This product includes software developed by Jens Vagelpohl
#for use in the Z Object Publishing Environment
#(http://www.zope.org/).

import sys
from os.path import join

from OFS.ObjectManager import ObjectManager,BeforeDeleteException
from Acquisition import aq_base,aq_chain
from AccessControl import getSecurityManager
from zLOG import LOG, ERROR
from ImageFile import ImageFile
from DateTime import DateTime

import NotificationsFolder
import Subscriber
from Utils import *
import Sentinel

SentinelID = Sentinel.SentinelID

def getZopeVersion():
    """ get ZOPE version from version.txt """
    zope_path = join(INSTANCE_HOME, 'lib/python/version.txt')
    version_file = open(zope_path, 'r')
    ZOPE_VERSION = string.split(version_file.read())[1]
    version_file.close()
    return ZOPE_VERSION

def initialize(context):
    """Change Notifications """

    #add Sentinel
    app = context._ProductContext__app
    global Sentinel

    if hasattr(app, SentinelID):
        Sentinel = getattr(app, SentinelID)
    else:
        try:
            oSentinel = Sentinel.Sentinel(id=SentinelID)
            app._setObject(SentinelID, oSentinel)
            get_transaction().note('Added Sentinel')
            get_transaction().commit()
        except:
            pass
        Sentinel = getattr(app, SentinelID)
    assert Sentinel is not None

    context.registerClass(
            NotificationsFolder.NotificationsFolder,
            permission='Add Notifications Folder',
            constructors=(NotificationsFolder.manage_addNotificationsFolderForm,
                                      NotificationsFolder.manage_addNotificationsFolder,
                                      ),
            icon='icons/NotificationsFolder.gif',
            )

    context.registerClass(
            Subscriber.Subscriber,
            permission='Add Subscriber for notifications',
            constructors=(Subscriber.manage_addSubscriberForm,
                                      Subscriber.manage_addSubscriber,
                                      ),
            icon='icons/Subscriber.gif',
            )

    context.registerHelp()
    context.registerHelpTitle('ChangeNotifications')

misc_ = {
        "Subscriber_jpg":  ImageFile("icons/Subscriber.gif", globals()),
        "Sentinel":  ImageFile("icons/Sentinel.gif", globals()),
}

#The following code "track" actions such add,copy/paste,cut/paste for all
#objects in order to send "immediate" notifications.

def manage_afterAdd(self, item, container):
    for object in self.objectValues():

        #begin modification
        try:
            newObj = computeRelativePath(self.absolute_url(1), id)
            oSentinel=self.unrestrictedTraverse(SentinelID, None)
            if oSentinel:
                for folder_url in oSentinel.getList():
                    try:
                        ofolder=self.unrestrictedTraverse(folder_url)
                        ofolder.addLink(DateTime(),newObj,object.meta_type)
                    except:
                        pass
        except:
            pass
        #end modification

        try: s=object._p_changed
        except: s=0
        if hasattr(aq_base(object), 'manage_afterAdd'):
            object.manage_afterAdd(item, container)
        if s is None: object._p_deactivate()

def manage_beforeDelete(self, item, container):
    for object in self.objectValues():

        #Begin modification
        #Update Sentinel list.
        try:
            if object.meta_type == 'Notifications folder':
                oSentinel = self.unrestrictedTraverse(SentinelID, None)
                if oSentinel:
                    oSentinel.delFromList(object.absolute_url(1))

        except:
            pass
        #End modification
        try: s=object._p_changed
        except: s=0
        try:
            if hasattr(aq_base(object), 'manage_beforeDelete'):
                object.manage_beforeDelete(item, container)
        except BeforeDeleteException, ob:
            raise
        except:
            LOG('Zope',ERROR,'manage_beforeDelete() threw',
                    error=sys.exc_info())
            pass
        if s is None: object._p_deactivate()

def _setObject(self,id,object,roles=None,user=None, set_owner=1,REQUEST=None):
    v=self._checkId(id)
    if v is not None: id=v
    try:    t=object.meta_type
    except: t=None

    # If an object by the given id already exists, remove it.
    for object_info in self._objects:
        if object_info['id'] == id:
            self._delObject(id)
            break

    self._objects=self._objects+({'id':id,'meta_type':t},)
    # Prepare the _p_jar attribute immediately. _getCopy() may need it.     
    #begin modification
    if getZopeVersion() == "2.5.1":
        if hasattr(object, '_p_jar') and object._p_jar is None:
            object._p_jar = self._p_jar
    self._setOb(id,object)
    object=self._getOb(id)

    try:
        newObj = computeRelativePath(self.absolute_url(1), id)
        oSentinel = self.unrestrictedTraverse(SentinelID, None)
        if oSentinel:
            for folder_url in oSentinel.getList():
                try:
                    ofolder=self.unrestrictedTraverse(folder_url)
                    ofolder.addLink(DateTime(), newObj, object)
                except:
                    pass
    except:
        pass
    #end modification

    if set_owner:
        object.manage_fixupOwnershipAfterAdd()

        # Try to give user the local role "Owner", but only if
        # no local roles have been set on the object yet.
        if hasattr(object, '__ac_local_roles__'):
            if object.__ac_local_roles__ is None:
                user=getSecurityManager().getUser()
                if user is not None:
                    name=user.getUserName()
                    if name != 'Anonymous User':
                        object.manage_setLocalRoles(name, ['Owner'])

    object.manage_afterAdd(object, self)
    return id

ObjectManager.manage_afterAdd=manage_afterAdd
ObjectManager.manage_beforeDelete=manage_beforeDelete
ObjectManager._setObject=_setObject